==> llm-docs-0.1.5.tgz: package/docs/api/graphql.md <==
---
id: api-graphql
---
# GraphQL API Guide

<!-- title: GraphQL API Guide -->
<!-- description: Guide for the LLM Platform Platform GraphQL API -->
<!-- lastUpdated: 2024-03-19 -->

## Overview

This guide details the GraphQL API implementation for the LLM Platform Platform. It covers schema definition, resolvers, mutations, subscriptions, and best practices.

## Architecture

### Core Components

1. **Schema**
   - Type definitions
   - Queries
   - Mutations
   - Subscriptions

2. **Resolvers**
   - Query resolvers
   - Mutation resolvers
   - Subscription resolvers
   - Field resolvers

3. **Services**
   - Business logic
   - Data access
   - Integration
   - Caching

## Implementation Guide

### Schema Definition

```typescript
// src/schema/schema.ts
export const typeDefs = gql`
  type Agent {
    id: ID!
    name: String!
    provider: String!
    model: String!
    config: AgentConfig
    status: AgentStatus!
    createdAt: DateTime!
    updatedAt: DateTime!
  }
  
  type AgentConfig {
    temperature: Float
    maxTokens: Int
    stopSequences: [String]
    systemPrompt: String
  }
  
  type AgentStatus {
    isActive: Boolean!
    lastUsed: DateTime
    usageCount: Int!
  }
  
  type Query {
    agents(
      filter: AgentFilter
      pagination: PaginationInput
    ): AgentConnection!
    agent(id: ID!): Agent
    models(
      filter: ModelFilter
      pagination: PaginationInput
    ): ModelConnection!
    model(id: ID!): Model
  }
  
  type Mutation {
    createAgent(input: CreateAgentInput!): Agent!
    updateAgent(id: ID!, input: UpdateAgentInput!): Agent!
    deleteAgent(id: ID!): Boolean!
    executeAgent(id: ID!, input: ExecuteAgentInput!): ExecutionResult!
  }
  
  type Subscription {
    agentStatusChanged(id: ID!): AgentStatus!
    executionProgress(id: ID!): ExecutionProgress!
  }
`;
```

### Base Resolver

```typescript
// src/resolvers/base.resolver.ts
export class BaseResolver {
  constructor(
    private normalizationService: NormalizationService,
    private errorNormalizer: ErrorNormalizerService
  ) {}
  
  protected async handleRequest<T>(
    handler: () => Promise<T>,
    schema: Schema
  ): Promise<NormalizedResponse<T>> {
    try {
      const data = await handler();
      return this.normalizeResponse(data, schema);
    } catch (error) {
      throw this.errorNormalizer.normalize(error);
    }
  }
  
  protected async normalizeResponse<T>(
    data: T,
    schema: Schema
  ): Promise<NormalizedResponse<T>> {
    return this.normalizationService.normalize(data, schema);
  }
}
```

### Agent Resolver

```typescript
// src/resolvers/agent.resolver.ts
@Resolver('Agent')
export class AgentResolver extends BaseResolver {
  constructor(
    private agentService: AgentService,
    normalizationService: NormalizationService,
    errorNormalizer: ErrorNormalizerService
  ) {
    super(normalizationService, errorNormalizer);
  }
  
  @Query()
  async agents(
    @Args('filter') filter: AgentFilter,
    @Args('pagination') pagination: PaginationInput
  ): Promise<NormalizedResponse<AgentConnection>> {
    return this.handleRequest(
      () => this.agentService.list(filter, pagination),
      agentConnectionSchema
    );
  }
  
  @Query()
  async agent(
    @Args('id') id: string
  ): Promise<NormalizedResponse<Agent>> {
    return this.handleRequest(
      () => this.agentService.get(id),
      agentSchema
    );
  }
  
  @Mutation()
  async createAgent(
    @Args('input') input: CreateAgentInput
  ): Promise<NormalizedResponse<Agent>> {
    return this.handleRequest(
      () => this.agentService.create(input),
      agentSchema
    );
  }
  
  @Mutation()
  async updateAgent(
    @Args('id') id: string,
    @Args('input') input: UpdateAgentInput
  ): Promise<NormalizedResponse<Agent>> {
    return this.handleRequest(
      () => this.agentService.update(id, input),
      agentSchema
    );
  }
  
  @Mutation()
  async deleteAgent(
    @Args('id') id: string
  ): Promise<NormalizedResponse<boolean>> {
    return this.handleRequest(
      () => this.agentService.delete(id),
      booleanSchema
    );
  }
  
  @Subscription()
  async agentStatusChanged(
    @Args('id') id: string
  ): AsyncIterator<AgentStatus> {
    return this.agentService.subscribeToStatus(id);
  }
}
```

### Model Resolver

```typescript
// src/resolvers/model.resolver.ts
@Resolver('Model')
export class ModelResolver extends BaseResolver {
  constructor(
    private modelService: ModelService,
    normalizationService: NormalizationService,
    errorNormalizer: ErrorNormalizerService
  ) {
    super(normalizationService, errorNormalizer);
  }
  
  @Query()
  async models(
    @Args('filter') filter: ModelFilter,
    @Args('pagination') pagination: PaginationInput
  ): Promise<NormalizedResponse<ModelConnection>> {
    return this.handleRequest(
      () => this.modelService.list(filter, pagination),
      modelConnectionSchema
    );
  }
  
  @Query()
  async model(
    @Args('id') id: string
  ): Promise<NormalizedResponse<Model>> {
    return this.handleRequest(
      () => this.modelService.get(id),
      modelSchema
    );
  }
}
```

## Integration Guide

### Authentication

```typescript
// src/guards/auth.guard.ts
@Injectable()
export class AuthGuard implements CanActivate {
  constructor(private authService: AuthService) {}
  
  async canActivate(context: ExecutionContext): Promise<boolean> {
    const gqlContext = GqlExecutionContext.create(context);
    const { req } = gqlContext.getContext();
    const token = this.extractToken(req);
    
    if (!token) {
      throw new UnauthorizedException();
    }
    
    try {
      const user = await this.authService.validateToken(token);
      req.user = user;
      return true;
    } catch (error) {
      throw new UnauthorizedException();
    }
  }
  
  private extractToken(req: Request): string {
    const auth = req.headers.authorization;
    if (!auth || !auth.startsWith('Bearer ')) {
      return null;
    }
    return auth.substring(7);
  }
}
```

### Rate Limiting

```typescript
// src/middleware/rate-limit.middleware.ts
@Injectable()
export class RateLimitMiddleware implements NestMiddleware {
  constructor(private rateLimitService: RateLimitService) {}
  
  async use(req: Request, res: Response, next: Function) {
    const key = this.getKey(req);
    const limit = await this.rateLimitService.check(key);
    
    res.setHeader('X-RateLimit-Limit', limit.limit);
    res.setHeader('X-RateLimit-Remaining', limit.remaining);
    res.setHeader('X-RateLimit-Reset', limit.reset);
    
    if (limit.remaining <= 0) {
      throw new TooManyRequestsException();
    }
    
    next();
  }
  
  private getKey(req: Request): string {
    return `${req.ip}:${req.body.operationName}`;
  }
}
```

### Caching

```typescript
// src/interceptors/cache.interceptor.ts
@Injectable()
export class CacheInterceptor implements NestInterceptor {
  constructor(private cacheService: CacheService) {}
  
  async intercept(
    context: ExecutionContext,
    next: CallHandler
  ): Promise<Observable<any>> {
    const gqlContext = GqlExecutionContext.create(context);
    const { req } = gqlContext.getContext();
    const key = this.getCacheKey(req);
    
    const cached = await this.cacheService.get(key);
    if (cached) {
      return of(cached);
    }
    
    return next.handle().pipe(
      tap(async (data) => {
        await this.cacheService.set(key, data);
      })
    );
  }
  
  private getCacheKey(req: Request): string {
    return `${req.body.operationName}:${JSON.stringify(req.body.variables)}`;
  }
}
```

## Best Practices

1. **Schema Design**
   - Use proper types
   - Define interfaces
   - Handle pagination
   - Support filtering

2. **Resolver Implementation**
   - Handle errors
   - Validate input
   - Use services
   - Cache results

3. **Security**
   - Authenticate requests
   - Authorize access
   - Rate limit
   - Validate input

4. **Performance**
   - Cache queries
   - Optimize resolvers
   - Use dataloaders
   - Monitor usage

## Related Documentation

- [OpenAPI Validation](openapi-validation)
- [API Reference](reference)
- [API Normalization](normalization)
- [REST API](rest)
- [Security Guide](../operations/security)
- [Development Guide](../development/README)

==> llm-docs-1.0.1.tgz: package/docs/api/graphql.md <==
---
id: api-graphql
---
# GraphQL API Guide

<!-- title: GraphQL API Guide -->
<!-- description: Guide for the Bluefly Platform GraphQL API -->
<!-- lastUpdated: 2024-03-19 -->

## Overview

This guide details the GraphQL API implementation for the Bluefly Platform. It covers schema definition, resolvers, mutations, subscriptions, and best practices.

## Architecture

### Core Components

1. **Schema**
   - Type definitions
   - Queries
   - Mutations
   - Subscriptions

2. **Resolvers**
   - Query resolvers
   - Mutation resolvers
   - Subscription resolvers
   - Field resolvers

3. **Services**
   - Business logic
   - Data access
   - Integration
   - Caching

## Implementation Guide

### Schema Definition

```typescript
// src/schema/schema.ts
export const typeDefs = gql`
  type Agent {
    id: ID!
    name: String!
    provider: String!
    model: String!
    config: AgentConfig
    status: AgentStatus!
    createdAt: DateTime!
    updatedAt: DateTime!
  }
  
  type AgentConfig {
    temperature: Float
    maxTokens: Int
    stopSequences: [String]
    systemPrompt: String
  }
  
  type AgentStatus {
    isActive: Boolean!
    lastUsed: DateTime
    usageCount: Int!
  }
  
  type Query {
    agents(
      filter: AgentFilter
      pagination: PaginationInput
    ): AgentConnection!
    agent(id: ID!): Agent
    models(
      filter: ModelFilter
      pagination: PaginationInput
    ): ModelConnection!
    model(id: ID!): Model
  }
  
  type Mutation {
    createAgent(input: CreateAgentInput!): Agent!
    updateAgent(id: ID!, input: UpdateAgentInput!): Agent!
    deleteAgent(id: ID!): Boolean!
    executeAgent(id: ID!, input: ExecuteAgentInput!): ExecutionResult!
  }
  
  type Subscription {
    agentStatusChanged(id: ID!): AgentStatus!
    executionProgress(id: ID!): ExecutionProgress!
  }
`;
```

### Base Resolver

```typescript
// src/resolvers/base.resolver.ts
export class BaseResolver {
  constructor(
    private normalizationService: NormalizationService,
    private errorNormalizer: ErrorNormalizerService
  ) {}
  
  protected async handleRequest<T>(
    handler: () => Promise<T>,
    schema: Schema
  ): Promise<NormalizedResponse<T>> {
    try {
      const data = await handler();
      return this.normalizeResponse(data, schema);
    } catch (error) {
      throw this.errorNormalizer.normalize(error);
    }
  }
  
  protected async normalizeResponse<T>(
    data: T,
    schema: Schema
  ): Promise<NormalizedResponse<T>> {
    return this.normalizationService.normalize(data, schema);
  }
}
```

### Agent Resolver

```typescript
// src/resolvers/agent.resolver.ts
@Resolver('Agent')
export class AgentResolver extends BaseResolver {
  constructor(
    private agentService: AgentService,
    normalizationService: NormalizationService,
    errorNormalizer: ErrorNormalizerService
  ) {
    super(normalizationService, errorNormalizer);
  }
  
  @Query()
  async agents(
    @Args('filter') filter: AgentFilter,
    @Args('pagination') pagination: PaginationInput
  ): Promise<NormalizedResponse<AgentConnection>> {
    return this.handleRequest(
      () => this.agentService.list(filter, pagination),
      agentConnectionSchema
    );
  }
  
  @Query()
  async agent(
    @Args('id') id: string
  ): Promise<NormalizedResponse<Agent>> {
    return this.handleRequest(
      () => this.agentService.get(id),
      agentSchema
    );
  }
  
  @Mutation()
  async createAgent(
    @Args('input') input: CreateAgentInput
  ): Promise<NormalizedResponse<Agent>> {
    return this.handleRequest(
      () => this.agentService.create(input),
      agentSchema
    );
  }
  
  @Mutation()
  async updateAgent(
    @Args('id') id: string,
    @Args('input') input: UpdateAgentInput
  ): Promise<NormalizedResponse<Agent>> {
    return this.handleRequest(
      () => this.agentService.update(id, input),
      agentSchema
    );
  }
  
  @Mutation()
  async deleteAgent(
    @Args('id') id: string
  ): Promise<NormalizedResponse<boolean>> {
    return this.handleRequest(
      () => this.agentService.delete(id),
      booleanSchema
    );
  }
  
  @Subscription()
  async agentStatusChanged(
    @Args('id') id: string
  ): AsyncIterator<AgentStatus> {
    return this.agentService.subscribeToStatus(id);
  }
}
```

### Model Resolver

```typescript
// src/resolvers/model.resolver.ts
@Resolver('Model')
export class ModelResolver extends BaseResolver {
  constructor(
    private modelService: ModelService,
    normalizationService: NormalizationService,
    errorNormalizer: ErrorNormalizerService
  ) {
    super(normalizationService, errorNormalizer);
  }
  
  @Query()
  async models(
    @Args('filter') filter: ModelFilter,
    @Args('pagination') pagination: PaginationInput
  ): Promise<NormalizedResponse<ModelConnection>> {
    return this.handleRequest(
      () => this.modelService.list(filter, pagination),
      modelConnectionSchema
    );
  }
  
  @Query()
  async model(
    @Args('id') id: string
  ): Promise<NormalizedResponse<Model>> {
    return this.handleRequest(
      () => this.modelService.get(id),
      modelSchema
    );
  }
}
```

## Integration Guide

### Authentication

```typescript
// src/guards/auth.guard.ts
@Injectable()
export class AuthGuard implements CanActivate {
  constructor(private authService: AuthService) {}
  
  async canActivate(context: ExecutionContext): Promise<boolean> {
    const gqlContext = GqlExecutionContext.create(context);
    const { req } = gqlContext.getContext();
    const token = this.extractToken(req);
    
    if (!token) {
      throw new UnauthorizedException();
    }
    
    try {
      const user = await this.authService.validateToken(token);
      req.user = user;
      return true;
    } catch (error) {
      throw new UnauthorizedException();
    }
  }
  
  private extractToken(req: Request): string {
    const auth = req.headers.authorization;
    if (!auth || !auth.startsWith('Bearer ')) {
      return null;
    }
    return auth.substring(7);
  }
}
```

### Rate Limiting

```typescript
// src/middleware/rate-limit.middleware.ts
@Injectable()
export class RateLimitMiddleware implements NestMiddleware {
  constructor(private rateLimitService: RateLimitService) {}
  
  async use(req: Request, res: Response, next: Function) {
    const key = this.getKey(req);
    const limit = await this.rateLimitService.check(key);
    
    res.setHeader('X-RateLimit-Limit', limit.limit);
    res.setHeader('X-RateLimit-Remaining', limit.remaining);
    res.setHeader('X-RateLimit-Reset', limit.reset);
    
    if (limit.remaining <= 0) {
      throw new TooManyRequestsException();
    }
    
    next();
  }
  
  private getKey(req: Request): string {
    return `${req.ip}:${req.body.operationName}`;
  }
}
```

### Caching

```typescript
// src/interceptors/cache.interceptor.ts
@Injectable()
export class CacheInterceptor implements NestInterceptor {
  constructor(private cacheService: CacheService) {}
  
  async intercept(
    context: ExecutionContext,
    next: CallHandler
  ): Promise<Observable<any>> {
    const gqlContext = GqlExecutionContext.create(context);
    const { req } = gqlContext.getContext();
    const key = this.getCacheKey(req);
    
    const cached = await this.cacheService.get(key);
    if (cached) {
      return of(cached);
    }
    
    return next.handle().pipe(
      tap(async (data) => {
        await this.cacheService.set(key, data);
      })
    );
  }
  
  private getCacheKey(req: Request): string {
    return `${req.body.operationName}:${JSON.stringify(req.body.variables)}`;
  }
}
```

## Best Practices

1. **Schema Design**
   - Use proper types
   - Define interfaces
   - Handle pagination
   - Support filtering

2. **Resolver Implementation**
   - Handle errors
   - Validate input
   - Use services
   - Cache results

3. **Security**
   - Authenticate requests
   - Authorize access
   - Rate limit
   - Validate input

4. **Performance**
   - Cache queries
   - Optimize resolvers
   - Use dataloaders
   - Monitor usage

## Related Documentation

- [OpenAPI Validation](openapi-validation)
- [API Reference](reference)
- [API Normalization](normalization)
- [REST API](rest)
- [Security Guide](../operations/security)
- [Development Guide](../development/README)
